searchData={"items":[{"type":"module","title":"Pathex","doc":"This module contains main functions and macros used to create, use and manipulate paths.","ref":"Pathex.html"},{"type":"module","title":"Usage - Pathex","doc":"To use Pathex just insert to your context. You can import Pathex in module body or even in function body.\n```elixir\nrequire Pathex\nimport Pathex, only: [path: 1, path: 2, \"~>\": 2, ...]\n```\n\nOr you can just `use Pathex`.\n```elixir\ndefmodule MyModule do\n\n  # `default_mod` option is optional\n  # when no mod is specified, `:naive` is selected\n  use Pathex, default_mod: :json\n\n  ...\nend\n```\nThis will import all operatiors and `path` macro","ref":"Pathex.html#module-usage"},{"type":"module","title":"Available macros - Pathex","doc":"Any macro here belongs to one of three categories:\n1. Macro which creates path closure (only `path/2`)\n2. Macro which uses path closure to manipulate the value (like `over/3`, `set/3`, `view/2`, ...)\n3. Macro which creates some path composition (like `alongside/1`, `~>/2`, `|||/2`, ...)","ref":"Pathex.html#module-available-macros"},{"type":"macro","title":"Pathex.&&&/2","doc":"Creates composition of two paths which has some inspiration from logical `and`.\nThis means that `a &&& b` path-closure tries to apply `a` and only if it returns `{:ok, something}`, tries\napply `b` and if `b` returns **exactly the same** as `a` does, the `a &&& b` returns `{:ok, something}`","ref":"Pathex.html#&&&/2"},{"type":"macro","title":"Example - Pathex.&&&/2","doc":"iex> p1 = path :x / :y\n    iex> p2 = path :a / :b\n    iex> ap = p1 &&& p2\n    iex> {:ok, 1} = view %{x: %{y: 1}, a: [b: 1]}, ap\n    iex> :error = view %{x: %{y: 1}, a: [b: 2]}, ap\n    iex> {:ok, %{x: %{y: 2}, a: [b: 2]}} = set %{x: %{y: 1}, a: [b: 1]}, ap, 2\n    iex> {:ok, %{x: %{y: 2}, a: %{b: 2}}} = force_set %{}, ap, 2","ref":"Pathex.html#&&&/2-example"},{"type":"macro","title":"Pathex.__using__/1","doc":"Easy and convenient way to add pathex to your module.\n\nYou can specify modifier\n```elixir\nuse Pathex, default_mod: :json\n```\n\nOr just use it with default `:naive` modifier\n```elixir\nuse Pathex\n```\n\n> #### `use Pathex` {: .info}\n>\n> When you `use Pathex`, the Pathex module will\n> require `Pathex` and import `Pathex`'s operators, `path/2` and `alongside/1` macros.\n> Plus it will set special module attribute with `default_mod` value in it.","ref":"Pathex.html#__using__/1"},{"type":"macro","title":"Pathex.alongside/1","doc":"This macro creates compositions of paths which work along with each other\n\nThink of `alongside([path1, path2, path3])` as `path1 &&& path2 &&& path3`\nThe only difference is that for viewing alongside returns list of variables","ref":"Pathex.html#alongside/1"},{"type":"macro","title":"Example - Pathex.alongside/1","doc":"iex> pa = alongside [path(:x), path(:y)]\n    iex> {:ok, [1, 2]} = view(%{x: 1, y: 2}, pa)\n    iex> {:ok, %{x: 3, y: 3}} = set(%{x: 1, y: 2}, pa, 3)\n    iex> :error = set(%{x: 1}, pa, 3)\n    iex> {:ok, %{x: 1, y: 1}} = force_set(%{}, pa, 1)","ref":"Pathex.html#alongside/1-example"},{"type":"macro","title":"Pathex.at/3","doc":"Applies `func` under `path` in `struct` and returns result of this `func`.","ref":"Pathex.html#at/3"},{"type":"macro","title":"Example - Pathex.at/3","doc":"iex> x = 1\n    iex> {:ok, 9} = at [0, %{x: 8}], path(x / :x), fn x -> x + 1 end\n    iex> p = path \"hey\" / 0\n    iex> {:ok, {:here, 9}} = at(%{\"hey\" => {9, -9}}, p, & {:here, &1})","ref":"Pathex.html#at/3-example"},{"type":"macro","title":"Pathex.at!/3","doc":"Applies `func` under `path` in `struct` and returns result of this `func`.\nRaises if path is not found.","ref":"Pathex.html#at!/3"},{"type":"macro","title":"Example - Pathex.at!/3","doc":"iex> x = 1\n    iex> 9 = at! [0, %{x: 8}], path(x / :x), fn x -> x + 1 end\n    iex> p = path \"hey\" / 0\n    iex> {:here, 9} = at!(%{\"hey\" => {9, -9}}, p, & {:here, &1})","ref":"Pathex.html#at!/3-example"},{"type":"macro","title":"Pathex.concat/2","doc":"The same as `Pathex.~>/2` for those who do not like operators","ref":"Pathex.html#concat/2"},{"type":"macro","title":"Example - Pathex.concat/2","doc":"iex> p1 = path :x / :y\n    iex> p2 = path :a / :b\n    iex> composed_path = concat(p1, p2)\n    iex> {:ok, 1} = view %{x: [y: [a: [a: 0, b: 1]]]}, composed_path","ref":"Pathex.html#concat/2-example"},{"type":"macro","title":"Pathex.delete/2","doc":"Deletes value under `path` in `struct`.","ref":"Pathex.html#delete/2"},{"type":"macro","title":"Example - Pathex.delete/2","doc":"iex> x = 1\n    iex> {:ok, [0, %{}]} = delete([0, %{x: 8}], path(x / :x))\n    iex> :error = delete([0, %{x: 8}], path(1 / :y))","ref":"Pathex.html#delete/2-example"},{"type":"macro","title":"Pathex.delete!/2","doc":"Deletes value under `path` in `struct` or raises if value is not found.","ref":"Pathex.html#delete!/2"},{"type":"macro","title":"Example - Pathex.delete!/2","doc":"iex> x = 1\n    iex> [0, %{}] = delete!([0, %{x: 8}], path(x / :x))","ref":"Pathex.html#delete!/2-example"},{"type":"macro","title":"Pathex.exists?/2","doc":"Gets the value under `path` in `struct` or returns default value if not found.","ref":"Pathex.html#exists?/2"},{"type":"macro","title":"Example - Pathex.exists?/2","doc":"iex> x = 1\n    iex> true = exists?([0, %{x: 8}], path(x / :x))\n    iex> p = path \"hey\" / \"you\"\n    iex> false = exists?(%{\"hey\" => [x: 1]}, p)","ref":"Pathex.html#exists?/2-example"},{"type":"macro","title":"Pathex.force_over/4","doc":"Applies `func` under `path` of `struct`.\n\nIf the path does not exist it creates the path favouring maps\nwhen structure is unknown and inserts default value.","ref":"Pathex.html#force_over/4"},{"type":"macro","title":"Example - Pathex.force_over/4","doc":"iex> x = 1\n    iex> {:ok, [0, %{x: {:xxx, 8}}]} = force_over([0, %{x: 8}], path(x / :x), & {:xxx, &1}, 123)\n    iex> p = path \"hey\" / 0\n    iex> {:ok, %{\"hey\" => %{0 => 1}}} = force_over(%{}, p, fn x -> x + 1 end, 1)\n\nIf the item in path doesn't have the right type, it returns `:error`.","ref":"Pathex.html#force_over/4-example"},{"type":"macro","title":"Example - Pathex.force_over/4","doc":"iex> p = path \"hey\" / \"you\"\n    iex> :error = force_over %{\"hey\" => {1, 2}}, p, fn x -> x end, \"value\"","ref":"Pathex.html#force_over/4-example"},{"type":"macro","title":"Pathex.force_over!/4","doc":"Applies `func` under `path` of `struct`.\n\nIf the path does not exist it creates the path favouring maps\nwhen structure is unknown and inserts default value.","ref":"Pathex.html#force_over!/4"},{"type":"macro","title":"Example - Pathex.force_over!/4","doc":"iex> x = 1\n    iex> [0, %{x: {:xxx, 8}}] = force_over!([0, %{x: 8}], path(x / :x), & {:xxx, &1}, 123)\n    iex> p = path \"hey\" / 0\n    iex> %{\"hey\" => %{0 => 1}} = force_over!(%{}, p, fn x -> x + 1 end, 1)\n\nIf the item in path doesn't have the right type, it raises.","ref":"Pathex.html#force_over!/4-example"},{"type":"macro","title":"Example - Pathex.force_over!/4","doc":"iex> p = path \"hey\" / \"you\"\n    iex> force_over! %{\"hey\" => {1, 2}}, p, fn x -> x end, \"value\"\n    ** (Pathex.Error) Type mismatch in structure","ref":"Pathex.html#force_over!/4-example"},{"type":"macro","title":"Pathex.force_set/3","doc":"Sets the `value` under `path` in `struct`.","ref":"Pathex.html#force_set/3"},{"type":"macro","title":"Creates path if it is not present - Pathex.force_set/3","doc":"If the path does not exist it creates the path favouring maps\nwhen structure is unknown.\n\n    iex> x = 1\n    iex> {:ok, [0, %{x: 123}]} = force_set [0, %{x: 8}], path(x / :x), 123\n    iex> p = path \"hey\" / 0\n    iex> {:ok, %{\"hey\" => %{0 => 1}}} = force_set %{}, p, 1","ref":"Pathex.html#force_set/3-creates-path-if-it-is-not-present"},{"type":"macro","title":"Incorrect types may be detected during call - Pathex.force_set/3","doc":"If the item in path doesn't have the right type, it returns `:error`.\n\n    iex> p = path \"hey\" / \"you\"\n    iex> :error = force_set %{\"hey\" => {1, 2}}, p, \"value\"","ref":"Pathex.html#force_set/3-incorrect-types-may-be-detected-during-call"},{"type":"macro","title":"Empty space is filled with nil - Pathex.force_set/3","doc":"Note that for paths created with `Pathex.path/2` list and tuple indexes\nwhich are out of bounds fill the empty space with `nil`.\n\n    iex> p = path 4\n    iex> {:ok, [1, 2, 3, nil, 5]} = force_set [1, 2, 3], p, 5\n    iex> {:ok, {1, 2, 3, nil, 5}} = force_set {1, 2, 3}, p, 5","ref":"Pathex.html#force_set/3-empty-space-is-filled-with-nil"},{"type":"macro","title":"Negative indexes - Pathex.force_set/3","doc":"This is also true for negative indexes (except -1 for lists which always prepends)\n\n    iex> p = path -5\n    iex> {:ok, [0, nil, 1, 2, 3]} = force_set [1, 2, 3], p, 0\n    iex> {:ok, {0, nil, 1, 2, 3}} = force_set {1, 2, 3}, p, 0","ref":"Pathex.html#force_set/3-negative-indexes"},{"type":"macro","title":"Pathex.force_set!/3","doc":"Sets the `value` under `path` in `struct`.","ref":"Pathex.html#force_set!/3"},{"type":"macro","title":"Creates path if it is not present - Pathex.force_set!/3","doc":"If the path does not exist it creates the path favouring maps\nwhen structure is unknown.\n\n    iex> x = 1\n    iex> [0, %{x: 123}] = force_set! [0, %{x: 8}], path(x / :x), 123\n    iex> p = path \"hey\" / 0\n    iex> %{\"hey\" => %{0 => 1}} = force_set! %{}, p, 1","ref":"Pathex.html#force_set!/3-creates-path-if-it-is-not-present"},{"type":"macro","title":"Incorrect types may be detected during call - Pathex.force_set!/3","doc":"If the item in path doesn't have the right type, it raises.\n\n    iex> p = path \"hey\" / \"you\"\n    iex> force_set! %{\"hey\" => {1, 2}}, p, \"value\"\n    ** (Pathex.Error) Type mismatch in structure","ref":"Pathex.html#force_set!/3-incorrect-types-may-be-detected-during-call"},{"type":"macro","title":"Empty space is filled with nil - Pathex.force_set!/3","doc":"Note that for paths created with `Pathex.path/2` list and tuple indexes\nwhich are out of bounds fill the empty space with `nil`.\n\n    iex> p = path 4\n    iex> [1, 2, 3, nil, 5] = force_set! [1, 2, 3], p, 5\n    iex> {1, 2, 3, nil, 5} = force_set! {1, 2, 3}, p, 5","ref":"Pathex.html#force_set!/3-empty-space-is-filled-with-nil"},{"type":"macro","title":"Negative indexes - Pathex.force_set!/3","doc":"This is also true for negative indexes (except `-1` for lists which always prepends)\n\n    iex> p = path -5\n    iex> [0, nil, 1, 2, 3] = force_set! [1, 2, 3], p, 0\n    iex> {0, nil, 1, 2, 3} = force_set! {1, 2, 3}, p, 0","ref":"Pathex.html#force_set!/3-negative-indexes"},{"type":"macro","title":"Pathex.get/3","doc":"Gets the value under `path` in `struct` or returns `default` when `path` is not present.\nNote that the default value is always lazily evaluted.","ref":"Pathex.html#get/3"},{"type":"macro","title":"Example - Pathex.get/3","doc":"iex> x = 1\n    iex> 8 = get([0, %{x: 8}], path(x / :x))\n    iex> p = path \"hey\" / \"you\"\n    iex> nil = get(%{\"hey\" => [x: 1]}, p)\n    iex> :default = get(%{\"hey\" => [x: 1]}, p, :default)","ref":"Pathex.html#get/3-example"},{"type":"function","title":"Pathex.inspect/1","doc":"Inspect the given path-closure and returns string which corresponds to given path-closure","ref":"Pathex.html#inspect/1"},{"type":"function","title":"Example - Pathex.inspect/1","doc":"iex> index = 1\n    iex> p = path(:x) ~> path(:y / index) &&& path(-1)\n    iex> Pathex.inspect(p)\n    \"path(:x) ~> path(:y / 1) &&& path(-1)\"","ref":"Pathex.html#inspect/1-example"},{"type":"macro","title":"Pathex.over/3","doc":"Applies `func` to the item under the `path` in `struct`\nand returns modified structure. Works like `Map.update!/3` but doesn't raise.","ref":"Pathex.html#over/3"},{"type":"macro","title":"Example - Pathex.over/3","doc":"iex> index = 1\n    iex> inc = fn x -> x + 1 end\n    iex> {:ok, [0, %{x: 9}]} = over [0, %{x: 8}], path(index / :x), inc\n    iex> p = path \"hey\" / 0\n    iex> {:ok, %{\"hey\" => [2, [2]]}} = over %{\"hey\" => [1, [2]]}, p, inc\n\n> Note:\n> Exceptions from passed function left unhandled\n    iex> over(%{1 => \"x\"}, path(1), fn x -> x + 1 end)\n    ** (ArithmeticError) bad argument in arithmetic expression","ref":"Pathex.html#over/3-example"},{"type":"macro","title":"Pathex.over!/3","doc":"Applies the `func` to the item under `path` in `struct` and returns modified structure.\nWorks like `Map.update!/3`.","ref":"Pathex.html#over!/3"},{"type":"macro","title":"Example - Pathex.over!/3","doc":"iex> x = 1\n    iex> inc = fn x -> x + 1 end\n    iex> [0, %{x: 9}] = over! [0, %{x: 8}], path(x / :x), inc\n    iex> p = path \"hey\" / 0\n    iex> %{\"hey\" => [2, [2]]} = over! %{\"hey\" => [1, [2]]}, p, inc","ref":"Pathex.html#over!/3-example"},{"type":"macro","title":"Pathex.path/2","doc":"Creates path from `quoted` ast. Paths look like unix filesystems paths and consist of\nelements separated from each other with `/`. Each element defines the key or index\nin the collection.","ref":"Pathex.html#path/2"},{"type":"macro","title":"Example - Pathex.path/2","doc":"iex> x = 1\n    iex> mypath = path 1 / :atom / \"string\" / {\"tuple?\"} / x\n    iex> structure = [0, [atom: %{\"string\" => %{{\"tuple?\"} => %{1 => 2}}}]]\n    iex> {:ok, 2} = view structure, mypath\n\nPaths can be used with one of the verbs in `Pathex` module (for example, `Pathex.view/2`).\nPaths can be customized with [modifiers](modifiers.md), composed using one of\ncomposition operators (`Pathex.concat/2`, `Pathex.~>/2`, `Pathex.|||/2`, `Pathex.&&&/2` or\n`Pathex.alongside/1`).\n\n> Note:\n> Each element in path can have collection type annotated using `::` operator. Available collection\n> types are `:list`, `:keyword`, `:tuple` and `:map`. Multiple collections can be annotated using list\n> It must comply with the limits set with [modifier](modifiers.md).","ref":"Pathex.html#path/2-example"},{"type":"macro","title":"Example - Pathex.path/2","doc":"iex> p = path( (0 :: [:list, :map]) / (:x :: :keyword) )\n    iex> {:ok, :hit} = view %{0 => [x: :hit]}, p\n    iex> {:ok, :hit} = view [[x: :hit]], p\n    iex> :error = view [%{x: :hit}], p","ref":"Pathex.html#path/2-example"},{"type":"macro","title":"Pathex.pattern/2","doc":"This macro converts path (which can be matched upon) into pattern.\n\nThese requirements must be satisfied in order for this macro to work correctly:\n1. Path must be inlined into this macro. This means that path must be defined\nin a argument of this macro\n2. Defined paths must contain constants only\n3. Path must result only in case with one clause","ref":"Pathex.html#pattern/2"},{"type":"macro","title":"Example - Pathex.pattern/2","doc":"iex> import Pathex\n    iex> structure = %{users: %{1 => %{fname: \"Jose\", lname: \"Valim\"}}}\n    iex> case structure do\n    ...>   pattern(fname, path(:users / 1 / :fname, :map)) ->\n    ...>     {:ok, fname}\n    ...>\n    ...>   _ ->\n    ...>     :error\n    ...> end\n    {:ok, \"Jose\"}","ref":"Pathex.html#pattern/2-example"},{"type":"macro","title":"Pathex.pop/2","doc":"Macro which gets value in the structure and deletes it.\n> Note:\n>\n> Current implementation of this function performs double lookup.\n> Which is still more efficient than `pop_in`","ref":"Pathex.html#pop/2"},{"type":"macro","title":"Example - Pathex.pop/2","doc":"iex> {:ok, {1, [2, 3]}} = pop([1, 2, 3], path(0))","ref":"Pathex.html#pop/2-example"},{"type":"macro","title":"Pathex.pop!/2","doc":"Gets value under `path` in `struct` and then deletes it.\n> Note:\n>\n> Current implementation of this function performs double lookup.\n> Which is still more efficient than `pop_in`","ref":"Pathex.html#pop!/2"},{"type":"macro","title":"Example - Pathex.pop!/2","doc":"iex> {1, [2, 3]} = pop!([1, 2, 3], path(0))","ref":"Pathex.html#pop!/2-example"},{"type":"macro","title":"Pathex.set/3","doc":"Sets `value` under `path` in `structure`. Think of it like `Map.put/3`.","ref":"Pathex.html#set/3"},{"type":"macro","title":"Example - Pathex.set/3","doc":"iex> x = 1\n    iex> {:ok, [0, %{x: 123}]} = set [0, %{x: 8}], path(x / :x), 123\n    iex> p = path \"hey\" / 0\n    iex> {:ok, %{\"hey\" => [123, [2]]}} = set %{\"hey\" => [1, [2]]}, p, 123","ref":"Pathex.html#set/3-example"},{"type":"macro","title":"Pathex.set!/3","doc":"Sets the `value` under `path` in `struct`. Think of it like `Map.put/3`.","ref":"Pathex.html#set!/3"},{"type":"macro","title":"Example - Pathex.set!/3","doc":"iex> x = 1\n    iex> [0, %{x: 123}] = set! [0, %{x: 8}], path(x / :x), 123\n    iex> p = path \"hey\" / 0\n    iex> %{\"hey\" => [123, [2]]} = set! %{\"hey\" => [1, [2]]}, p, 123","ref":"Pathex.html#set!/3-example"},{"type":"macro","title":"Pathex.view/2","doc":"Gets the value under `path` in `struct`.","ref":"Pathex.html#view/2"},{"type":"macro","title":"Example - Pathex.view/2","doc":"iex> x = 1\n    iex> {:ok, 8} = view [0, %{x: 8}], path(x / :x)\n    iex> p = path \"hey\" / 0\n    iex> {:ok, 9} = view %{\"hey\" => {9, -9}}, p","ref":"Pathex.html#view/2-example"},{"type":"macro","title":"Pathex.view!/2","doc":"Gets the value under `path` in `struct`. Raises if `path` not found.","ref":"Pathex.html#view!/2"},{"type":"macro","title":"Example - Pathex.view!/2","doc":"iex> x = 1\n    iex> 8 = view! [0, %{x: 8}], path(x / :x)\n    iex> p = path \"hey\" / 0\n    iex> 9 = view! %{\"hey\" => {9, -9}}, p","ref":"Pathex.html#view!/2-example"},{"type":"macro","title":"Pathex.|||/2","doc":"Creates composition of two paths which has some inspiration from logical `or`.\nThis means that `a ||| b` path-closure tries to apply `a` and only if it returns `:error`, tries\napply `b`","ref":"Pathex.html#%7C%7C%7C/2"},{"type":"macro","title":"Example - Pathex.|||/2","doc":"iex> p1 = path :x / :y\n    iex> p2 = path :a / :b\n    iex> op = p1 ||| p2\n    iex> {:ok, 1} = view %{x: %{y: 1}, a: [b: 2]}, op\n    iex> {:ok, 2} = view %{x: 1, a: [b: 2]}, op\n    iex> {:ok, %{x: %{y: 2}, a: [b: 1]}} = set %{x: %{y: 1}, a: [b: 1]}, op, 2\n    iex> {:ok, %{x: %{y: 2}}} = force_set %{}, op, 2\n    iex> {:ok, %{x: %{}, a: [b: 1]}} = force_set %{x: %{y: 1}, a: [b: 1]}, op, 2","ref":"Pathex.html#%7C%7C%7C/2-example"},{"type":"macro","title":"Pathex.~>/2","doc":"Creates composition of two paths similar to concatenating them together.\nThis means that `a ~> b` path-closure applies `a` and only if it returns `{:ok, something}`\nit applies `b` to `something`","ref":"Pathex.html#~%3E/2"},{"type":"macro","title":"Example - Pathex.~>/2","doc":"iex> p1 = path :x / :y\n    iex> p2 = path :a / :b\n    iex> composed_path = p1 ~> p2\n    iex> {:ok, 1} = view %{x: [y: [a: [a: 0, b: 1]]]}, composed_path","ref":"Pathex.html#~%3E/2-example"},{"type":"type","title":"Pathex.force_update_args/2","doc":"","ref":"Pathex.html#t:force_update_args/2"},{"type":"type","title":"Pathex.inner_func/1","doc":"Function which is passed to path-closure as second element in args tuple","ref":"Pathex.html#t:inner_func/1"},{"type":"type","title":"Pathex.inspect_args/0","doc":"","ref":"Pathex.html#t:inspect_args/0"},{"type":"type","title":"Pathex.mod/0","doc":"More about [modifiers](modifiers.md)","ref":"Pathex.html#t:mod/0"},{"type":"type","title":"Pathex.pathex_compatible_structure/0","doc":"This depends on the modifier","ref":"Pathex.html#t:pathex_compatible_structure/0"},{"type":"type","title":"Pathex.result/1","doc":"Value returned by non-bang path call","ref":"Pathex.html#t:result/1"},{"type":"type","title":"Pathex.t/0","doc":"Also known as [path-closure](path.md)","ref":"Pathex.html#t:t/0"},{"type":"type","title":"Pathex.t/2","doc":"Also known as [path-closure](path.md)","ref":"Pathex.html#t:t/2"},{"type":"type","title":"Pathex.update_args/2","doc":"","ref":"Pathex.html#t:update_args/2"},{"type":"module","title":"Pathex.Accessibility","doc":"Helpers to create paths from `Access.t()` and lists\n\n> Note:\n> This module is separate because functions presented in this module\n> are suboptimal. You should try use `Pathex.path/2` first, and only\n> if its not applicable to you use-case, you should use functions\n> from this module","ref":"Pathex.Accessibility.html"},{"type":"function","title":"Pathex.Accessibility.from_access/1","doc":"Creates path-closure from `Access.t()`.\n\n> Note:\n> Paths created using this function do not support force operations yet","ref":"Pathex.Accessibility.html#from_access/1"},{"type":"function","title":"Example - Pathex.Accessibility.from_access/1","doc":"iex> import Pathex\n    iex> p = from_access [:x, :y]\n    iex> 10 = view!(%{x: [y: 10]}, p)","ref":"Pathex.Accessibility.html#from_access/1-example"},{"type":"function","title":"Pathex.Accessibility.from_list/2","doc":"Creates path from list of items. The list of items should not be known\nat runtime, therefore some optimizations of paths are not possible. Use this\nfunction only when `Pathex.path/2` is not applicable","ref":"Pathex.Accessibility.html#from_list/2"},{"type":"function","title":"Example - Pathex.Accessibility.from_list/2","doc":"iex> import Pathex\n    iex> p = from_list [:x, 1, :y]\n    iex> 10 = view!(%{x: [1, [y: 10]]}, p)","ref":"Pathex.Accessibility.html#from_list/2-example"},{"type":"macro","title":"Pathex.Accessibility.from_record/3","doc":"Creates a lens from record macro.\nAll arguments of this macro must be compile time atoms","ref":"Pathex.Accessibility.html#from_record/3"},{"type":"macro","title":"Example - Pathex.Accessibility.from_record/3","doc":"iex> require Pathex\n    iex> defmodule User do\n    ...>   import Record; defrecord(:user, name: \"\", age: nil)\n    ...>\n    ...>   def new(name) do\n    ...>     name_lens = from_record(User, :user, :name)\n    ...>     Pathex.force_set!({}, name_lens, name)\n    ...>   end\n    ...> end\n    iex> {:user, \"Joe\", nil} = User.new(\"Joe\")","ref":"Pathex.Accessibility.html#from_record/3-example"},{"type":"function","title":"Pathex.Accessibility.from_struct/2","doc":"Creates a lens for specific key in the structure. It behaves the same way as a lens created\nwith `Pathex.path/2` but it works only with specified structure and `force_set` creates\na structure with default values.","ref":"Pathex.Accessibility.html#from_struct/2"},{"type":"function","title":"Example - Pathex.Accessibility.from_struct/2","doc":"iex> require Pathex\n    iex> defmodule User do\n    ...>   defstruct name: \"\", age: nil\n    ...> end\n    iex> name_lens = from_struct(User, :name)\n    iex> %{name: \"Joe\"} = Pathex.force_set!(%{}, name_lens, \"Joe\")","ref":"Pathex.Accessibility.html#from_struct/2-example"},{"type":"function","title":"Pathex.Accessibility.to_access/1","doc":"Converts path-closure to `Access.t()`","ref":"Pathex.Accessibility.html#to_access/1"},{"type":"function","title":"Example - Pathex.Accessibility.to_access/1","doc":"iex> import Pathex\n    iex> access = to_access path(:x / 0, :map)\n    iex> 1 = get_in(%{x: %{0 => 1}}, access)","ref":"Pathex.Accessibility.html#to_access/1-example"},{"type":"type","title":"Pathex.Accessibility.access/0","doc":"","ref":"Pathex.Accessibility.html#t:access/0"},{"type":"module","title":"Pathex.Combinator","doc":"Combinator for lenses\nRead `Pathex.Combinator.combine/1` documentation","ref":"Pathex.Combinator.html"},{"type":"function","title":"Pathex.Combinator.combine/2","doc":"This function creates a recursive path from path defined in `path_func`\nConsider this example\n\n    iex> import Pathex; import Pathex.Lenses\n    iex> recursive_xpath = combine(fn recursive_xpath ->\n    iex>   path(:x)            # Takes by :x key\n    iex>   ~> recursive_xpath  # If taken, calls itself\n    iex>   ||| matching(_)     # Otherwise returns current structure\n    iex> end)\n    iex>\n    iex> Pathex.view!(%{x: %{x: %{x: %{x: 1}}}}, recursive_xpath)\n    1\n    iex> Pathex.set!(%{x: %{x: %{x: %{x: 1}}}}, recursive_xpath, 2)\n    %{x: %{x: %{x: %{x: 2}}}}\n\nThe second argument of this function specifies the maximum depth. It's infinity be default,\nbut you can specify this as any positive integer. It is useful when you're developing lens\nand you're not sure whether the lens will or won't loop.\n\nFor example\n```elixir\n# Combinator lens with limit\nlimited = combine(fn rec -> path(:x) ~> rec end, 100_000)\n:error = Pathex.force_set(%{x: 1}, limited, 123)\n\n# And this is without limit\nunlimited = combine(fn rec -> path(:x) ~> rec end)\nPathex.force_set(%{x: 1}, unlimited, 123) # Infinite loop\n```","ref":"Pathex.Combinator.html#combine/2"},{"type":"module","title":"Pathex.Debug","doc":"Special lens which logs all operation\nand performs the same way `matching(_)` does","ref":"Pathex.Debug.html"},{"type":"function","title":"Pathex.Debug.debug/1","doc":"Works like `matching(_)` lens, but also prints out debugging information with prefix.","ref":"Pathex.Debug.html#debug/1"},{"type":"function","title":"Example - Pathex.Debug.debug/1","doc":"iex> p = path(1) ~> debug(\"debug()> \")\n  iex> view! %{1 => 2}, p\n  debug()> Called view on 2\n  2","ref":"Pathex.Debug.html#debug/1-example"},{"type":"module","title":"Pathex.Lenses","doc":"Module with collection of prebuilt paths","ref":"Pathex.Lenses.html"},{"type":"function","title":"Pathex.Lenses.all/0","doc":"Path function which works with **all** possible keys it can find\nIt takes all keys **and than** applies inner function (or concated path)\nIf any application fails, this lens returns `:error`","ref":"Pathex.Lenses.html#all/0"},{"type":"function","title":"Example - Pathex.Lenses.all/0","doc":"iex> require Pathex; import Pathex\n    iex> alll = Pathex.Lenses.all()\n    iex> Pathex.over!([%{x: 0}, [x: 1]], alll ~> path(:x), fn x -> x + 1 end)\n    [%{x: 1}, [x: 2]]\n    iex> Pathex.view!(%{x: 1, y: 2, z: 3}, alll) |> Enum.sort()\n    [1, 2, 3]\n    iex> Pathex.set([x: 1, y: 0], alll, 2)\n    {:ok, [x: 2, y: 2]}","ref":"Pathex.Lenses.html#all/0-example"},{"type":"function","title":"Pathex.Lenses.any/0","doc":"Path function which works with **any** possible key it can find\nIt takes any key **and than** applies inner function (or concated path)","ref":"Pathex.Lenses.html#any/0"},{"type":"function","title":"Example - Pathex.Lenses.any/0","doc":"iex> require Pathex\n    iex> anyl = Pathex.Lenses.any()\n    iex> {:ok, 1} = Pathex.view %{x: 1}, anyl\n    iex> {:ok, [9]} = Pathex.set  [8], anyl, 9\n    iex> {:ok, [x: 1, y: 2]} = Pathex.force_set [x: 0, y: 2], anyl, 1\n\nNote that force setting value to empty map has undefined behaviour\nand therefore returns an error:\n    iex> require Pathex\n    iex> anyl = Pathex.Lenses.any()\n    iex> :error = Pathex.force_set(%{}, anyl, :well)\n\nAnd note that this lens has keywords at head of list at a higher priority\nthan non-keyword heads:\n    iex> require Pathex\n    iex> anyl = Pathex.Lenses.any()\n    iex> {:ok, [{:x, 1}, 2]} = Pathex.set([{:x, 0}, 2], anyl, 1)\n    iex> {:ok, [1, {:x, 2}]} = Pathex.set([0, {:x, 2}], anyl, 1)\n    iex> {:ok, [1, 2]} = Pathex.set([{\"some_tuple\", \"here\"}, 2], anyl, 1)","ref":"Pathex.Lenses.html#any/0-example"},{"type":"function","title":"Pathex.Lenses.filtering/1","doc":"This macro creates path-closure successes only when `predicate` returns truthy value.\n`predicate` is a function which takes a structures upon which the path is called and\nreturns a boolean (or any other type which will be treated as boolean).\n\nThis function is useful when composed with `star/0` and `some/0`\n\n> Note:  \n> In terms of functional programming, such conditional lenses are called prisms","ref":"Pathex.Lenses.html#filtering/1"},{"type":"function","title":"Example - Pathex.Lenses.filtering/1","doc":"iex> import Pathex.Lenses; import Pathex\n    iex> adminl = filtering(& &1.role == :admin)\n    iex> {:ok, %{name: \"Name\", role: :admin}} = Pathex.view(%{name: \"Name\", role: :admin}, adminl)\n    iex> :error = Pathex.view(%{role: :user}, adminl)\n\n    iex> import Pathex.Lenses; import Pathex\n    iex> dots2d = [{1, 1}, {1, 5}, {3, 0}, {4, 3}]\n    iex> higher_than_2 = filtering(fn {_x, y} -> y > 2 end)\n    iex> {:ok, [{1, 5}, {4, 3}]} = Pathex.view(dots2d, star() ~> higher_than_2)","ref":"Pathex.Lenses.html#filtering/1-example"},{"type":"macro","title":"Pathex.Lenses.matching/1","doc":"This macro creates path-closure which successes only when input matches the `pattern`.\nThe `pattern` can be just a pattern or a pattern with `when`. You can write this patterns\njust like you'd write them in `case`\n\nThis function is useful when composed with `star/0` and `some/0`\n\n> Note:  \n> In terms of functional programming, such conditional lenses are called prisms","ref":"Pathex.Lenses.html#matching/1"},{"type":"macro","title":"Example - Pathex.Lenses.matching/1","doc":"iex> import Pathex.Lenses; import Pathex\n    iex> adminl = matching(%{role: :admin})\n    iex> {:ok, %{name: \"Name\", role: :admin}} = Pathex.view(%{name: \"Name\", role: :admin}, adminl)\n    iex> :error = Pathex.view(%{}, adminl)\n\n    iex> import Pathex.Lenses; import Pathex\n    iex> dots2d = [{1, 1}, {1, 5}, {3, 0}, {4, 3}]\n    iex> higher_than_2 = matching({_x, y} when y > 2)\n    iex> {:ok, [{1, 5}, {4, 3}]} = Pathex.view(dots2d, star() ~> higher_than_2)","ref":"Pathex.Lenses.html#matching/1-example"},{"type":"function","title":"Pathex.Lenses.some/0","doc":"Path function which applies inner function (or concated path-closure)\nto the first value it can apply it to","ref":"Pathex.Lenses.html#some/0"},{"type":"function","title":"Example - Pathex.Lenses.some/0","doc":"iex> require Pathex; import Pathex\n    iex> somel = Pathex.Lenses.some()\n    iex> Pathex.view!([x: [11], y: [22], z: 33], somel ~> path(0))\n    11\n    iex> Pathex.set!([x: %{y: 0}, z: %{y: 0}], somel ~> path(:y, :map), 1)\n    [x: %{y: 1}, z: %{y: 0}]\n    iex> Pathex.view([x: 1, y: 2, z: 3], somel)\n    {:ok, 1}\n\n> Note:  \n> Force update fails for empty structures\n\nThink of this function as `star() ~> any()` but optimized to work with only first element","ref":"Pathex.Lenses.html#some/0-example"},{"type":"function","title":"Pathex.Lenses.star/0","doc":"Path function which applies inner function (or concated path-closure)\nto every value it can apply it to","ref":"Pathex.Lenses.html#star/0"},{"type":"function","title":"Example - Pathex.Lenses.star/0","doc":"iex> require Pathex; import Pathex\n    iex> starl = Pathex.Lenses.star()\n    iex> Pathex.view!(%{x: [1], y: [2], z: 3}, starl ~> path(0)) |> Enum.sort()\n    [1, 2]\n    iex> Pathex.set!(%{x: %{y: 0}, z: [3]}, starl ~> path(:y, :map), 1)\n    %{x: %{y: 1}, z: [3]}\n    iex> Pathex.view([x: 1, y: 2, z: 3], starl)\n    {:ok, [1, 2, 3]}\n\n> Note:  \n> It returns :error when no data was found or changed\n\nThink of this function as `filter_map`. It is particularly useful for filtering  \nand selecting needed values with custom functions or `matching/1` macro","ref":"Pathex.Lenses.html#star/0-example"},{"type":"function","title":"Example - Pathex.Lenses.star/0","doc":"iex> require Pathex; import Pathex; require Pathex.Lenses\n    iex> starl = Pathex.Lenses.star()\n    iex> structure = [{1, 4}, {2, 8}, {3, 6}, {4, 10}]\n    iex> #\n    iex> # For example we want to select all tuples with first element greater than 2\n    iex> #\n    iex> greater_than_2 = Pathex.Lenses.matching({x, _} when x > 2)\n    iex> Pathex.view(structure, starl ~> greater_than_2)\n    {:ok, [{3, 6}, {4, 10}]}","ref":"Pathex.Lenses.html#star/0-example"},{"type":"module","title":"Pathex.Short","doc":"This module provides short definitions of pathex paths.\n\nFor example, when using `Pathex.Short`, you can transform this\n```elixir\npath :x / :y / 1 / 2\n```\n\nTo just this\n```elixir\n:x / :y / 1 / 2\n```","ref":"Pathex.Short.html"},{"type":"macro","title":"Pathex.Short.//2","doc":"This macro redefines `/` operator, so use this macro with caution\nor only in bounded context (for example, you can `use Pathex.Short` only\ninside functions)","ref":"Pathex.Short.html#//2"},{"type":"macro","title":"Example - Pathex.Short.//2","doc":"iex> use Pathex.Short\n    iex> path = :x / :y\n    iex> 1 = Pathex.view!(%{x: %{y: 1}}, path)","ref":"Pathex.Short.html#//2-example"},{"type":"exception","title":"Pathex.Error","doc":"Simple exception for bang! functions (like `Pathex.view!/2`) errors.\n\n> Note:  \n> Some new field may be added in the future","ref":"Pathex.Error.html"},{"type":"type","title":"Pathex.Error.t/0","doc":"","ref":"Pathex.Error.html#t:t/0"},{"type":"extras","title":"Pathex","doc":"# Pathex\n\nSpeed or composability? Choose both!","ref":"readme.html"},{"type":"extras","title":"Elixir 1.17 warning - Pathex","doc":"It turns out that Elixir 1.17 release has 3 bugs which block reliable usage of Pathex library, one of which is critical.\nSo please, do not use Elixir 1.17 with the Pathex library. However, these issues are fixed in upcoming Elixir 1.18 release.","ref":"readme.html#elixir-1-17-warning"},{"type":"extras","title":"What is Pathex? - Pathex","doc":"Pathex is a library for performing fast actions with nested data structures in Elixir.\nWith pathex you can trivially set, get and update values in structures in a functional manner.\nIt provides all necessary logic to manipulate data structures in different ways using flexible __functional lens__ pattern.","ref":"readme.html#what-is-pathex"},{"type":"extras","title":"Why another library? - Pathex","doc":"Existing methods of accesssing data in nested structures are either slow (like `Focus`) or do not provide enough functionality (like `Access`). For example setting the value in structure with Pathex is `70-160x` faster than `Focus` or `2-3x` faster than `put_in` and `get_in`\n\n> You can checkout benchmarks at https://github.com/hissssst/pathex_bench","ref":"readme.html#why-another-library"},{"type":"extras","title":"Usage - Pathex","doc":"Pathex is really simple and straightforward to use (almost like `Enum`). You don't need to learn any specific language, just create paths with `path` and use verbs with them.","ref":"readme.html#usage"},{"type":"extras","title":"Add it to your module - Pathex","doc":"```elixir\n# This will import path macro and operators and require Pathex\nuse Pathex\n```\n\n> Or just `import Pathex`","ref":"readme.html#add-it-to-your-module"},{"type":"extras","title":"Create the path - Pathex","doc":"```elixir\npath_to_streets = path :user / :private / :addresses / 0 / :street\npath_in_json = path \"users\" / 1 / \"street\", :json\n```\n\nThis creates closure which can get, set, update and delete values in this path","ref":"readme.html#create-the-path"},{"type":"extras","title":"Use the path - Pathex","doc":"```elixir\n{:ok, \"6th avenue\" = street} =\n  %{\n    user: %{\n      id: 1,\n      name: \"hissssst\",\n      private: %{\n        phone: \"123-456-789\",\n        addresses: [\n           [city: \"City\", street: \"6th avenue\", mail_index: 123456]\n        ]\n      }\n    }\n  }\n  |> Pathex.view(path_to_streets)\n\n%{\n  \"users\" => %{\n    1 => %{\"street\" => \"6th avenue\"}\n  }\n} = Pathex.force_set!(%{}, path_in_json, street)\n```","ref":"readme.html#use-the-path"},{"type":"extras","title":"Features - Pathex","doc":"Pathex has a lot of different features and can even compete with code written by hand in terms of efficiency.\nPathex significantly reduces the time to write a code which manipulates nested structure, while\nproviding efficiency and composability. No more functions like `get_users`, `set_users`, `update_users`! No more XPaths, JSONPaths, CSS Selectors!","ref":"readme.html#features"},{"type":"extras","title":"Easy to use - Pathex","doc":"It's not harder to use than `Map` or `Enum`! Check out the [cheatsheet](https://hexdocs.pm/pathex/cheatsheet.html) for common tasks.\n\nPathex also provides more information about errors than any other tool.\n\n```elixir\niex(1)> field = :email\niex(2)> Pathex.view!(%{}, path(:users) ~> all() ~> path(:personal / field))\n** (Pathex.Error)\n  Couldn't find element\n\n    Path:      path(:users) ~> all() ~> path(:personal / :email)\n\n    Structure: %{}\n```","ref":"readme.html#easy-to-use"},{"type":"extras","title":"Fast - Pathex","doc":"Paths are just a set of pattern-matching cases.\nThis is done to extract maximum efficiency from BEAM's pattern-matching compiler.\n\n```elixir\n# Code for viewing variables for path\npath(1 / \"y\", :map)\n\n# Almost equals to\ncase input do\n  %{1 => %{\"y\" => res}} ->\n    {:ok, res}\n\n  _ ->\n    :error\nend\n```","ref":"readme.html#fast"},{"type":"extras","title":"Reusable - Pathex","doc":"One path can be used to update, get, set, delete or update a value in the structure!\nAnd these paths can even be composed together.\nThis composition is very efficient, there's no need to concatenate lists like `Access` does.\n\n```elixir\n# User structure\nuser = %User{\n  personal: %{fname: \"Kabs\", sname: \"Rocks\"},\n  phone: \"123-456-789\"\n}\n\n# Path to username in user structure\nusername = path(:personal / :fname)\n\n# Get a username\n{:ok, \"Kabs\"} = Pathex.view(user, username)\n\n# Set a username\nanother_user =\n  %User{\n    personal: %{fname: \"Blabs\", sname: \"Rocks\"},\n    phone: \"123-456-789\"\n  } = Pathex.set!(user, username, \"Blabs\")\n\n# Get all usernames!\nimport Pathex.Lenses\n[\"Kabs\", \"Blabs\"] =\n  [\n    user,\n    another_user\n  ]\n  |> Pathex.view!(all() ~> username)\n```\n\nPathex can be used to manipulate different nested data structures. From `GenServer` state to HTML or Elixir's AST!","ref":"readme.html#reusable"},{"type":"extras","title":"Extensible - Pathex","doc":"Pathex is built around simple primitive called `path-closure`, which is a simple closure with clearly defined specification. Anything complying with `Pathex.t()` spec can be used within `Pathex`.","ref":"readme.html#extensible"},{"type":"extras","title":"Installation - Pathex","doc":"```elixir\ndef deps do\n  [\n    {:pathex, \"~> 2.0\"}\n  ]\nend\n```\n\n> Pathex is significantly faster with OTP 26","ref":"readme.html#installation"},{"type":"extras","title":"Contributions - Pathex","doc":"Welcome! If you want to get your hands dirty, you can check existing `TODO`'s.\n\n> **By the way**\n>\n> If you have any suggestions or want to change something in this library don't hesitate to open an issue. If you have any whitepapers about functional lenses, you can add them in a PR to the bottom of this readme","ref":"readme.html#contributions"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"2.6.0 - Changelog","doc":"**Breaking**\n\nAll of these are bug fixes, that's why we only bump the minor version\n\n* Fixes bug with `-1` incorrectly appending when used in nested lenses\n* Now error is raised when modifier conflicts with path definition\n\n**Non-breaking**\n\n* Slightly improved docs\n* Added docs for Elixir 1.17","ref":"changelog.html#2-6-0"},{"type":"extras","title":"2.5.3 - Changelog","doc":"**Breaking**\n\nNone\n\n**Non-breaking**\n\nExceptions generated by incorrect modifiers produce more meaningful error messages","ref":"changelog.html#2-5-3"},{"type":"extras","title":"2.5.0 - Changelog","doc":"**Breaking**\n\nNone\n\n**Non-breaking**\n\n* `Pathex.Accessibility.from_struct/2` to create `force_over`-friendly lenses for Elixir structures\n* `Pathex.Accessibility.from_record/3` to create `force_over`-friendly lenses for records","ref":"changelog.html#2-5-0"},{"type":"extras","title":"2.4.3 - Changelog","doc":"* Documentation improvements, typos etc.\n* Documentation for VSCode reformatting","ref":"changelog.html#2-4-3"},{"type":"extras","title":"2.4.2 - Changelog","doc":"* Fixed `:delete` op bug for `all` lens #21\n* Improved performance of `all` lens\n* Added Code of Conduct","ref":"changelog.html#2-4-2"},{"type":"extras","title":"2.4.1 - Changelog","doc":"* Fixed `:delete` op bug for `all` lens #20","ref":"changelog.html#2-4-1"},{"type":"extras","title":"2.4 - Changelog","doc":"Negative indexes, lens fixes, force_update nil filling and refactoring\n\n**Breaking**\n\n* matching force_update fix. Now it doesn't raise an error when called\n* filtering force_update fix. Now it doesn't raise an error when called\n* Tuple force_update fix. Now it works with indexes and for `force_set {}, path(2), 1` it will create `{nil, nil, 1}`.\n* List force_update fix. Now it works with indexes and for `force_set [], path(2), 1` it will create `[nil, nil, 1]`.\n\n**Non-breaking**\n\n* Tuple negative index is now supported for all operations\n* List negative indexes now work the same way as positive indexes (but backwards), except `-1` which still prepends to the list.","ref":"changelog.html#2-4"},{"type":"extras","title":"2.3 - Changelog","doc":"**Breaking**\n\n* Pathex now uses Elixir 1.13 (or higher) version\n\n**Non-breaking**\n\n* `Pathex.t` spec fix\n* `Pathex.Short` for shorter path definition\n* `Pathex.pattern` for creating patterns from inlined paths\n* `Pathex` now can be `use`-d inside functions or anything like this\n* Paths inlining is now detected for aliased, imported and macro calls\n* Internal spec fixes, dead code eliminations and formatting","ref":"changelog.html#2-3"},{"type":"extras","title":"2.2 - Changelog","doc":"**Breaking**\n\n* Changed behaviour of path-closures created with `path` when working with improper `Keyword.t()`\n\n**Non-breaking**\n\n* Improved documentation\n* Removed dead code\n* Added `Pathex.Accessibility` module for creating paths in runtime","ref":"changelog.html#2-2"},{"type":"extras","title":"2.1 - Changelog","doc":"**Breaking**\n\n* Updating and viewing keywords with `star` and `some` lenses now doesn't raise when they're used against non-proper keyword\n\n**Non-breaking**\n\n* Concatenated paths now can force_over for not only maps\n* Fixed debug lens\n* Added ability to pass calls and arbitary structures into `path` macro\n* Unrolled some clauses for `star` and `some` for extra efficiency","ref":"changelog.html#2-1"},{"type":"extras","title":"2.0 - Changelog","doc":"**Breaking**\n\n* Reworked `star` lens. Now it is less optimistic and returns `:error` when no values were viewed/updated\n* Removed sigils\n* Removed deprecated lens `id`\n* Removed deprecated lens `either`\n* Removed `recur` function\n\n**Non-breaking**\n\n* `compose` function for recursive lens\n* `delete` method for all paths, lenses and higher order functions\n* `inspect` method for all paths, lenses and higher order functions\n* Matchable updater for lists and maps\n* Builders are selected for combination (not for mod as they used to)\n* Reworked documentation\n* Annotated paths","ref":"changelog.html#2-0"},{"type":"extras","title":"1.3.0 - Changelog","doc":"**Breaking**\n\nNone! __(See deprecated in Non-breaking)__\n\n**Non-breaking**\n\n* Deprecated `id` lens\n* Deprecated `either` lens\n\n* Fixed bug with concatenation context overlapping\n* `some` lens\n* `star` lens\n* `matching` lens\n* `filtering` lens\n* Removed some dead code\n* Moved lenses code to separate modules","ref":"changelog.html#1-3-0"},{"type":"extras","title":"1.2.0 - Changelog","doc":"**Breaking**\n\nNone!\n\n**Non-breaking**\n\n* `star` lens\n* `all` lens","ref":"changelog.html#1-2-0"},{"type":"extras","title":"1.1.0 - Changelog","doc":"**Breaking**\n\nNone!\n\n**Non-breaking**\n\n* `|||` operator\n* stack-optimized version of `~>` operator\n* stack-optimized version of `|||` operator","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.0 - Changelog","doc":"**Breaking**\n\n* `force_set`/`get`/`set` clause in closure was renamed to `force_update`/`view`/`update`\nand added a special argument with default value in it\n\n**Non-breaking**\n\n* `alongside` macro\n* stack-optimized version of `&&&` operator\n* path code generation size assertion\n* better documentation format\n* `id` lens\n* `either` lens\n* `any` lens\n\n> Yeah, it starts from 1.0.0\n> I can describe previous versions if anybody needs this. Just open an issue! :)","ref":"changelog.html#1-0-0"},{"type":"extras","title":"Basics","doc":"# Basics\n\nThis guide will show the basics of `Pathex`. This is the best place to start learning `Pathex`.\nIt won't take more than 5 minutes to read.\n\nAs [README](README.md) says, Pathex is a library for working with collections and nested key-value structures.\nPathex uses a powerful functional abstraction called a __lens__ (in Pathex we call it __path__ or __path-closure__, you'll later see why).\nPath is a basically a path to a value in a nested structure, almost like filesystem path.\nWith this path you can __use__ some verbs to get, set, delete or update the value in the structure.","ref":"basics.html"},{"type":"extras","title":"Create paths - Basics","doc":"First of all, we need to know how to create paths. A simple path can be created with the `Pathex.path/2` macro.","ref":"basics.html#create-paths"},{"type":"extras","title":"For example: - Basics","doc":"```elixir\np = path :x / :y\n# Note the \"/\". That's why the library is called Pathex\n```\n\nThis path specifies a way to get (or set, or delete or whatever) value `1` from `%{x: %{y: 1}}`\n\n> Note:\n> You **don't** have to pin variables used in paths\n>\n> ```elixir\n> x = 1\n> path :some_atom / x  # Right\n> path :some_atom / ^x # Wrong\n> ```\n\nEach type of value used in a `path` corresponds to one or more types of Elixir collections\n\n* Strings, tuples, maps, functions, lists are treated only as `Map` keys.\n* Atoms are treated as `Map` or `Keyword` keys.\n* Positive integers are treated as `Map` keys or `List` and `Tuple` indexes\n* Negative integers are treated as `Map` keys or first element in `List`'s view or\n  prepending element in `List`'s `update` or `force_set` operations\n* Variables are treated the same way as the type they have in runtime. For example, variables containing negative\n  integers are treated as negative integer values. Unfortunately, it is not possible to know variable type at compile-time,\n  that's why `Pathex` generates code for all possible types. See [performance tips](basics.md#performance-tips) for more information.","ref":"basics.html#for-example"},{"type":"extras","title":"Use paths - Basics","doc":"Alright, we have a path. Now, we need to know how to use it.","ref":"basics.html#use-paths"},{"type":"extras","title":"For example: - Basics","doc":"Let's take a value from nested structure\n\n```elixir\niex> first_username = path :usernames / 0\niex> Pathex.view(%{usernames: [\"SubZero\", \"Scorpion\"]}, first_username)\n{:ok, \"SubZero\"}\n```\n\nOr let's change the value\n```elixir\niex> Pathex.set(%{usernames: [\"SubZero\", \"Scorpion\"]}, first_username, \"Shao Khan\")\n{:ok, %{usernames: [\"Shao Khan\", \"Scorpion\"]}}\n```\n\nAs you can see, every macro with the verbal name in `Pathex` module is a macro for using the path.\nThere are a lot of other ways to use paths.\nIn all such macros, the first argument is an input structure and the second argument is a path-closure.\n\n> Note:\n> Path-closure which are created inside path-using macros are optimised to have\n> only one operation generated (instead of default three).\n> You can read more about path-closures and operations [here](path.md)\n\nAs usual\n\n* Macros without `!` always return `{:ok, result}` or `:error`\n* Macros with `!` return `result` or raise `Pathex.Error`","ref":"basics.html#for-example"},{"type":"extras","title":"Compose paths - Basics","doc":"Path-closures can be composed together to create new path-closure,\nevery path composition macro is a binary operator. Some compositions\nare optimized to generate one closure even if multiple closures\nare used.\n\nYou can concat paths with `Pathex.~>/2` composition macro","ref":"basics.html#compose-paths"},{"type":"extras","title":"For example: - Basics","doc":"```elixir\niex> p1 = path :x\niex> p2 = path :y\niex> composed_path = p1 ~> p2\niex> 1 = Pathex.view!(%{x: [y: 1]}, composed_path)\n```\n\n> You can think about path composition just like it's path concatenation in shell\n>\n> **For example:**\n>\n> ```elixir\n> iex> first_user = path :users / 0          # users/0\n> iex> name = path :name                     # name\n> iex> first_user_name = first_user ~> name  # users/0/name\n> ```","ref":"basics.html#for-example"},{"type":"extras","title":"Prebuilt paths - Basics","doc":"`Pathex` provides some prebuilt paths for non-standard data manipulation. You\ncan find them in `Pathex.Lenses` module. You can read more about them in\n[lenses guide](lenses.md)\n\nFor example `Pathex.Lenses.star/0` lens works just like `*` in filesystem path.","ref":"basics.html#prebuilt-paths"},{"type":"extras","title":"Performance tips - Basics","doc":"General rule: the more data you provide to `Pathex` at compile-time the better\n\nFor example:\n\n```elixir\npath(1 / :x / :y, :json)\n# works faster than\npath(1 / :x / :y)\n```\n\nBecause `:json` mod optimizes the closure to one big case,\nwhile the default `:naive` mod generates nested cases.\n\nAnd, in this example:\n\n```elixir\npath(1 / :x / :y)\n# works faster than\nx = :x; path(1 / x / :y)\n```\n\nBecause constants provide more information about available types.\nThis means that for `:x` Pathex knows that this can be a `Keyword` or `Map`,\nwhile `x` means that this is a variable which could contain a `List`/`Tuple` index or a `Map`/`Keyword` value\n\nAnd, in this example:\n\n```elixir\npath(1 / x / :y)\n# works faster than\npath(1) ~> path(x) ~> path(:y)\n```\n\nBecause path concatenation actually creates a path that calls all operands internally\nwhich increases the call stack and makes concatenated paths handle errors manually.","ref":"basics.html#performance-tips"},{"type":"extras","title":"Cheatsheet","doc":"# Cheatsheet\n\nThis documentation page is intended to provide some common use cases of `Pathex`.","ref":"cheatsheet.html"},{"type":"extras","title":"Operations - Cheatsheet","doc":"This section describes actions which can be used with path-closures.\nNote that for 1-depth operations `Pathex` is slightly slower than `Map` or `Keyword`\n\n| Map              | Pathex                |\n|:-----------------|----------------------:|\n| `Map.fetch/2`    | `Pathex.view/2`       |\n| `Map.get/3`      | `Pathex.get/3`        |\n| `Map.update!/3`  | `Pathex.over/3`       |\n| `Map.update/4`   | `Pathex.force_over/4` |\n| `Map.put/3`      | `Pathex.force_set/3`  |\n| `Map.replace/3`  | `Pathex.set/3`        |\n| `Map.delete/2`   | `Pathex.delete/2`     |\n| `Map.has_key?/2` | `Pathex.exists?/2`    |","ref":"cheatsheet.html#operations"},{"type":"extras","title":"Nested - Cheatsheet","doc":"This section describes how `Pathex` can be used instead of `Access` for accessing nested data structures.\nNote that `Pathex` is 2 to 4 times faster than `Access`\n\n| Access                             | Pathex                                      |\n|:-----------------------------------|--------------------------------------------:|\n| `get_in(s, [:x, :y, :z])`          | `Pathex.get(s, path(:x / :y / :z))`         |\n| `put_in(s, [:x, :y, :z], v)`       | `Pathex.set(s, path(:x / :y / :z), v)`      |\n\nNote that `Pathex` works with structs (i.e. `%User{}`) as maps and doesn't need any special behaviour implemented in the module. Plus `Pathex` works with tuples and lists.\n\nThis means that this code\n```\nstructure\n|> get_in([:x, :y, :z])\n|> Enum.at(10)\n|> elem(1)\n|> get_in([:z, :y, :x])\n```\n\nCan be rewritten to\n```\nPathex.view(structure, path(:x / :y / :z / 10 / 1 / :z / :y / :x))\n```","ref":"cheatsheet.html#nested"},{"type":"extras","title":"Enumerables and lenses - Cheatsheet","doc":"| Enum                | Pathex                 |\n|:--------------------|-----------------------:|\n| `Enum.find/2`       | `Pathex.Lenses.some/0` |\n| `Enum.map/2`        | `Pathex.Lenses.all/0`  |\n| `Enum.filter_map/3` | `Pathex.Lenses.star/0` |\n| `Enum.at/2`         | `Pathex.path/2`        |\n| `Enum.fetch/2`      | `Pathex.path/2`        |\n\n[More examples in Lenses tutorial](lenses.md)","ref":"cheatsheet.html#enumerables-and-lenses"},{"type":"extras","title":"Traverse leaves - Cheatsheet","doc":"This function traverses all leaves in the structure.\n\n```elixir\nuse Pathex; import Pathex.Combinator; import Pathex.Lenses\n\ndef leaves(iterlens \\\\ star()) do\n  combine(fn recursive ->\n    iterlens ~> (recursive ||| matching(_))\n  end)\nend\n\n[2, 1, [:dot, 1234]] =\n  %{\n    x: 1,\n    y: 2,\n    meta: %{\n      type: :dot,\n      id: 1234\n    }\n  }\n  |> Pathex.view!(leaves)\n```\n\nYou can change the `star()` lens to whatever lens you prefer.\nFor example, for parsing HTML documents you can use `star() ~> path(2)` to\nnot traverse attributes. And if you want to find one leaf, you can use `some()`","ref":"cheatsheet.html#traverse-leaves"},{"type":"extras","title":"Walk structure - Cheatsheet","doc":"If you want to walk the whole structure, not only leaves, but the structure and\nit's substructures too, you can use this function\n\n```elixir\nuse Pathex; import Pathex.Combinator; import Pathex.Lenses\n\n# Like Macro.postwalk but for any tree-like structure\ndef postwalking(iterlens, predicate) do\n  combine(fn recursive ->\n    predicate\n    ~> (\n      alongside([\n        iterlens ~> recursive,\n        matching(_)\n      ])\n      ||| matching(_)\n    )\n    ||| (iterlens ~> recursive)\n  end)\nend\n\n# Like Macro.prewalk but for any tree-like structure\ndef prewalking(iterlens, predicate) do\n  combine(fn recursive ->\n    predicate\n    ~> (\n      alongside([\n        matching(_),\n        iterlens ~> recursive\n      ])\n      ||| matching(_)\n    )\n    ||| (iterlens ~> recursive)\n  end)\nend\n\nwalking = postwalking(star(), matching(%{}))\n\n# This code updates all maps and submaps in the structure\n%{\n  size: 3,\n  x: 1,\n  y: 2,\n  meta: %{\n    type: :dot,\n    id: 1234,\n    size: 3,\n    empty_map_in_list: [[[%{size: 0}]]]\n  }\n} =\n  %{\n    x: 1,\n    y: 2,\n    meta: %{\n      type: :dot,\n      id: 1234,\n      empty_map_in_list: [[[%{}]]]\n    }\n  }\n  |> Pathex.over!(walking, & Map.put(&1, :size, map_size(&1)))\n```","ref":"cheatsheet.html#walk-structure"},{"type":"extras","title":"Lenses tutorial","doc":"# Lenses tutorial\n\nThis guide will show you how to create powerful lenses using `Pathex.Lenses` module. Here we will take a look at common tasks in nested data structure manipulation which can be solved using `Pathex.Lenses`.","ref":"lenses.html"},{"type":"extras","title":"For all values in collection - Lenses tutorial","doc":"What if we need to update all values in the collection matching specific pattern?\n\nThis simple task can be solved using Elixir's `Enum` module but it is quite tough\nto be polymorphic and reusable for different patterns or types of collections\n\nLet's say we have a list of users with roles and we want to add access to admin\npage for all admins:\n\n```elixir\n# The list looks like this\nusers = [\n  %{fname: \"John\", sname: \"Doe\", role: \"CEO\",   access: [\"admin_page\", \"users_page\"]},\n  %{fname: \"Mike\", sname: \"Lee\", role: \"admin\", access: [\"users_page\"]},\n  %{fname: \"Fred\", sname: \"Can\", role: \"admin\", access: [\"users_page\"]},\n  %{fname: \"Dave\", sname: \"Lee\", role: \"user\",  access: []}\n]\n```\n\nWith `Enum` this would look like\n\n```elixir\nnew_users =\n  Enum.map(users, fn\n    %{role: \"admin\", access: access} = user ->\n      %{user | access: Enum.uniq([\"admin_page\" | access])}\n\n    other ->\n      other\n  end)\n```\n\nBut using `Pathex.Lenses.star/0` and `Pathex.Lenses.matching/1` this would look like\n\n```elixir\nimport Pathex\nimport Pathex.Lenses\n\n# `l` in the end stands for `lens`\nadminl = matching(%{role: \"admin\"})\naccessl = path(:access)\n\n# `star()` works like `*` in shell.\n# so `star() ~> path(:file)` works almost like `*/file`\n\n# Here `star() ~> adminl` translates to `select * where role == \"admin\"`\n# and `star() ~> adminl ~> accessl` translates to `select access where role == \"admin\"`\nnew_users = Pathex.over!(users, star() ~> adminl ~> accessl, & Enum.uniq([\"admin_page\" | &1]))\n```","ref":"lenses.html#for-all-values-in-collection"},{"type":"extras","title":"For any value in collection - Lenses tutorial","doc":"What if we need to update first value matching specific pattern\n(in our example it will be `{:option, _}`) and we need to\nreturn `{:ok, updated_collection}` if the\nfirst value was updated and `:error` if not\n\nThis task can be also done using `Enum`, but what if we can write the solution  \nwhich would be as simple as saying `Update first value in collection, which matches the pattern`?\n\nWith `Enum` this would look really terrible.\nI couldn't come up with a polymorphic solution that would fit in less than 20 lines of code.\n\nBut with `Pathex.Lenses.some/0` and `Pathex.Lenses.matching/1` this would be as simple as\n\n```elixir\nuse Pathex; import Pathex.Lenses\ndef update_first_option(collection, update_func) do\n  Pathex.over(collection, some() ~> matching({:option, _}), update_func)\nend\n```","ref":"lenses.html#for-any-value-in-collection"},{"type":"extras","title":"For any value in nested structure - Lenses tutorial","doc":"Allright, we have a nested structure with various types inside and we need to find any value in any map\nfor which the special condition occurs and change it\n\nThink of an HTML-like structure without attributes like\n```elixir\n{\"html\", [\n  {\"head\", [...]},\n  {\"body\", [...]}\n  ]}\n```\n\nAnd we need to update __just one__ `label` with string which ends with `\"Please click subscribe button\"`\n\nIn Elixir we'd need to write a recursive function, which would untrivially update tuples and lists\n\nUsing `Pathex.Combinator.combine/1`, `Pathex.Lenses.some/0` and `Pathex.Lenses.filtering/1` it's very simple\n\n```elixir\nuse Pathex; import Pathex.Lenses; import Pathex.Lenses.Recur\n\npath_to_subscribe =\n  combine(fn recursive -> some() ~> (recursive() ||| matching()) end)\n  ~> matching({\"label\", _}) # To find a label\n  ~> path(1)               # To get to value of a label\n  ~> filtering(& String.ends_with?(&1, \"Please click subscribe button\")\n\nPathex.set(document, path_to_subscribe, \"Do not subscribe, hehe\")\n```","ref":"lenses.html#for-any-value-in-nested-structure"},{"type":"extras","title":"Recurring - Lenses tutorial","doc":"Sometimes you may want to create a recurring lens. We have a `Pathex.Combinator.combine/1` which can help you. It is a fixed point combinator for paths, what may sound too smart, but this is a function which makes your lens be able to compose with itself.\n\nConsider this representation of an HTML\n```\n{\"html\", [], [\n  {\"body\", [], ...}\n  {\"html\", [], ...}]}\n```\n\nAnd we want to update a node which has `id=big-image`. It would be nice to have a lens which would do something like\n```elixir\npath(2) ~> some()\n  ~> path(2) ~> some()\n    ~> path(2) ~> some()`\n```\nUntil it finds the tag we're looking for.\n\nMeet the `Pathex.Combinator.combine/1`. Using this higher order function we can represent lens combining with itself.\nThe lens above can be specified with this code.\n```elixir\ncombine(fn recursive ->\n  path(2) ~> some() ~> recursive\nend)\n```\n\nAnd we also need to find a specific id, so let's combine this lens with `matching(_)`\n```elixir\npath_to_id =\n  combine(fn recursive ->\n    path(2) ~> some ~> recursive\n  end)\n  ~> matching({_, [{\"id\", \"big-image\"}], _})\n```\n\nHowever, it has a drawback -- this recursive lens will never succeed, because we didn't specify the exit. Let's fix it\n```elixir\npath_to_big_image =\n  combine(fn recursive ->\n    path(2) ~> some ~> (recursive ||| matching(_))\n  end)\n  ~> matching({_, [{\"id\", \"big-image\"}], _})\n\n{\"div\", [{\"id\", \"big-image\"}], [\n  {\"img\", [{\"src\", \"/image.png\"}], \"\"}]} = Pathex.view!(html, path_to_big_image)\n```","ref":"lenses.html#recurring"},{"type":"extras","title":"Modifiers tutorial","doc":"# Modifiers tutorial\n\nEvery pathex `path` created with `Pathex.path/2` can have modifier specified as a second argument.\nModifier defines behaviour of the path in a way of structures it can match inside.\nFor example, path created with `:map` modifier can only match maps inside them.\nModifiers can be specified only in form of an atom, variables are not accepted.","ref":"modifiers.html"},{"type":"extras","title":"When? How? & Why? - Modifiers tutorial","doc":"You should use modifiers when you need to specify type of inner structures to match\nor reduce amount of generated code by `Pathex` or improve performance of the path","ref":"modifiers.html#when-how-why"},{"type":"extras","title":"Usage - Modifiers tutorial","doc":"Currently only three modifiers are available:\n\n* `:json` which matches lists and maps\n* `:naive` which matches lists, tuples, keywords and maps\n* `:map` which matches only maps\n\n> Default modifier for every path is `:naive`\n\nModifiers are specified as second argument in `path/2` like\n\n```elixir\npath :x / :y, :naive\npath 0 / :x, :json\n```","ref":"modifiers.html#usage"},{"type":"extras","title":"Naive modifier - Modifiers tutorial","doc":"This modifier matches lists, tuples, keyword and maps\nIt generates matches for every structure like","ref":"modifiers.html#naive-modifier"},{"type":"extras","title":"For example: - Modifiers tutorial","doc":"`path(:x, :naive)` generates something like\n\n```elixir\ncase input do\n  %{x: value} ->\n     ...\n  [{a, _} | _] = k when is_atom(a) ->\n     case Keyword.fetch(k, :x) do\n       ...\n     end\nend\n```","ref":"modifiers.html#for-example"},{"type":"extras","title":"Json modifier - Modifiers tutorial","doc":"This modifier specifies paths which macth lists (for integer keys only) and maps\n\n> Note:\n> This modifier treats variables as map keys, this means that\n>\n> ```elixir\n> iex> x = 1\n> iex> p = path x, :json\n> iex> :error = Pathex.view([1, 2, 3], p)\n> iex> {:ok, :x} = Pathex.view(%{1 => :x}, p)\n> ```\n\nBut passed integers are exanded into list matching\nthis makes it very efficient to view data from the structure","ref":"modifiers.html#json-modifier"},{"type":"extras","title":"For example: - Modifiers tutorial","doc":"`path 1 / :x, :json` generates closure with\n\n```elixir\ncase input do\n  [_, %{x: value} | _] ->\n    {:ok, value}\n\n  %{1 => %{x: value}} ->\n    {:ok, value}\n\n  _ ->\n    :error\nend\n```\n\nWhich extracts maximum efficiency from BEAM's pattern-matching","ref":"modifiers.html#for-example"},{"type":"extras","title":"Map modifier - Modifiers tutorial","doc":"This modifier matches only maps and therefore is the fastest modifier available","ref":"modifiers.html#map-modifier"},{"type":"extras","title":"For example: - Modifiers tutorial","doc":"`path 1 / :x / \"y\", :map` will generate closure with\n\n```elixir\ncase input do\n  %{1 => %{x: %{\"y\" => value}}} ->\n    {:ok, value}\n\n  _ ->\n    :error\nend\n```","ref":"modifiers.html#for-example"},{"type":"extras","title":"Annotations - Modifiers tutorial","doc":"Usually a key passed to `path` can match to one or more types. For example, key `:x` can be a key in `Map` like `%{x: 1}` and a key in `Keyword` like `[x: 1]`. To make key match only certain type it can be annotated to the exact type using this syntax `path(:x :: :map)`. Available annotations are `:map`, `:keyword`, `:list` and `:tuple`.","ref":"modifiers.html#annotations"},{"type":"extras","title":"For example: - Modifiers tutorial","doc":"```elixir\nfirst_in_tuple = path(1 :: :tuple)\n:error   = Pathex.view([0, 1, 2, 3], first_in_tuple)\n{:ok, 0} = Pathex.view({0, 1, 2, 3}, first_in_tuple)\n```","ref":"modifiers.html#for-example"},{"type":"extras","title":"Path","doc":"# Path\n\n> Note:  \n> This is a documentation about internal Pathex API and it is subject to change  \n> You do not need to read this unless you are interested in `Pathex` internals, want to hack `Pathex` or want to create your own `Pathex` compatible lens\n\nThis page describes what a `Pathex.t()` is and how to create and use one.\nMost of the time you might want to use `Pathex.path/2` and `Pathex.Lenses` to create paths.\nIf you find their functionality limited and unapplicable for your use case, you can create you own Pathex-compatible closure. This doc describes how to do this.","ref":"path.html"},{"type":"extras","title":"Path-closure - Path","doc":"Path-closure is specified in `Pathex.t()`, and it's a closure of two arguments:\n\n* Operation name. It is an atom, one of `:view`, `:update`, `:force_update`, `:delete`, `:inpsect`\n* Operation arguments. It is a tuple which size depends on an operation\n\nCurrently every path-closure has 5 operations:\n```elixir\npath_closure =\n  fn\n    # Operation which gets value from structure and return `function.(value)`\n    :view,         {structure, hook_function} -> ...\n\n    # Operation which returns new structure with updated value\n    :update,       {structure, hook_function} -> ...\n\n    # Operation which returns new structure with updated value, or default set\n    :force_update, {structure, hook_function, default} -> ...\n\n    # Operation which returns new structure with deleted value.\n    :delete,       {structure, delete_function} -> ...\n\n    # Inspects the path. Returns Elixir's AST. This is used only for error-logging and debugging\n    :inspect,      _ -> ...\n  end\n```\n\nHere `structure` is the structure which is viewed or updated by this path and `hook_function` or `delete_function` is a hook function described in the next section. If the value in the structure defined by the path is not present, path-closure **must** return `:error`. If it's present, path-closure must return whatever is returned by `hook_function(value)`","ref":"path.html#path-closure"},{"type":"extras","title":"Hook function - Path","doc":"You can see that for every operation except `inspect` accepts some function as a second argument. This function is called a hook function and it **must be called** on the value from the `structure` defined by the path (if the value is present). Hook function is required to","ref":"path.html#hook-function"},{"type":"extras","title":"Return types - Path","doc":"Here `function` returns `{:ok, result} | :error`  \nAnd `delete_function` returns `{:ok, result} | :error | :delete_me`\n\n* `{:ok, result}` returns updated value for update/force_update/delete operations and value to be returned for `view`\n* `:error` in case function call has not succeeded\n* `:delete_me` is returned by function **only** for `:delete` operation clause. It means that the value upon which the hook function was called must be deleted. For all other clauses, this must be treated as an invalid hook function and error **must** be raised","ref":"path.html#return-types"},{"type":"extras","title":"Qualities - Path","doc":"Special requirements are described here\n\n* Path-closure **must not** raise or throw if it's called with correct operation and argument tuple\n\n* Path-closure **must** be idempotent. This means that path-closure must return the same result for the same inputs every time it's called.\n\n* Path-closure **should** not produce any side-effects. Thought it actually can produce side-effects, you shouldn't count on them.","ref":"path.html#qualities"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}